Option Strict On
Option Explicit On

Imports System.IO.Ports

Friend Class frmMain

    Private Sub frmMain_Closed(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles MyBase.Closed

        Try
            'Cerramos el RS232 si esta abierto
            If SerialPort1.IsOpen Then SerialPort1.Close()
            'Cerramos el USB
            Call Close_USB()

        Catch ex As Exception
            Call HandleException(Me.Name, ex)
        End Try
    End Sub


    Private Sub frmMain_Load(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles MyBase.Load
        Try
            MyHID = New Hid()
            ' Caption del programa
            Me.Text = "Controlador 28 servos (USB/RS232) v1.0"

            ' LinkLabel
            LinkLabel1.Text = "Pulsa aqui para mas info."
            LinkLabel1.Links.Add(6, 4, "http://www.jvmbots.com/viewtopic.php?t=431")

            'Ocultamos los controles que no funcionan
            TrackBar29.Visible = False
            TrackBar30.Visible = False
            Label29.Visible = False
            Label30.Visible = False
            L29.Visible = False
            L30.Visible = False

            'Inicializamos el timer de TimeOut
            TimerOut.Interval = 300 '300 ms de timeout
            TimerOut.Enabled = False

            ' Cargamos el modo de comunicacion (USB=false, RS232=True)
            ModoRS232 = CBool(Val(mINI.IniGet(ArchivoIni, "Settings", "Modo", "0")))

            ' Segun el modo de comunicacion selecciona los items
            If ModoRS232 Then
                RadioButton1.Checked = False    'Usb
                RadioButton2.Checked = True     'Rs232
                Call ModuloRs232.AbrirPuerto()  'Abrimos el puerto serie
            Else
                RadioButton1.Checked = True     'Usb
                RadioButton2.Checked = False    'Rs232
                Call AbrirUSB(True)     'Buscamos la controladora
            End If

            ' Iniciamos el posicionado de controles
            Call Iniciar(Panel1)
            Call Iniciar(Panel2)
            Call Iniciar(Panel3)
            ' Leemos los datos de la controladora
            Call LeerPosiciones()
        Catch ex As Exception
            Call HandleException(Me.Name, ex)
        End Try

    End Sub















    Shared Sub HandleException(ByVal moduleName As String, ByVal e As Exception)

        Dim Message As String
        Dim Caption As String

        Try
            'Creamos el mensaje de error.
            Message = "Excepcion: " & e.Message & ControlChars.CrLf & _
            "Modulo: " & moduleName & ControlChars.CrLf & _
            "Metodo: " & e.TargetSite.Name

            'El caption.
            Caption = "Unexpected Exception"

            'Presentamos el mensaje en la ventana.
            MessageBox.Show(Message, Caption, MessageBoxButtons.OK)
            Debug.Write(Message)

        Finally
        End Try

    End Sub


    Protected Overrides Sub Finalize()
        MyBase.Finalize()
    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        Try
            ' Segun el modo enviamos el comando ALL al modulo USB o al Rs232
            If ModoRS232 = True Then
                Call ModuloRs232.Posicion_All_Servos()
            Else
                Call ModuloUSB.Posicion_All_Servos()
            End If


        Catch ex As Exception
            Call HandleException(Me.Name, ex)
        End Try

    End Sub

    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        Try
            Dim c As Integer
            ' Segun el modo enviamos el comando ALL al modulo USB o al Rs232
            If ModoRS232 = True Then
                'Enviamos la posicion de inicio a los 28 servos al RS232
                For c = 0 To 27
                    Call ModuloRs232.Comando_Inicio(c, Servo(c))
                Next c
            Else
                'Enviamos la posicion de inicio a los 28 servos al USB
                For c = 0 To 27
                    Call ModuloUSB.Comando_Inicio(c, Servo(c))
                Next c
            End If
        Catch ex As Exception
            Call HandleException(Me.Name, ex)
        End Try

    End Sub


    Private Sub Button3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button3.Click
        Try
            'Mostramos el formulario de configuracion
            FrmPuerto.ShowDialog()

        Catch ex As Exception
            Call HandleException(Me.Name, ex)
        End Try
    End Sub





    Private Sub Iniciar(ByVal container As Control)
        Try
            Dim Ctrl As TrackBar
            ' Inicializamos los faders de control
            For Each Ctrl In container.Controls
                If TypeOf (Ctrl) Is TrackBar Then
                    Ctrl.Minimum = 1
                    Ctrl.Maximum = 255
                    Ctrl.TickFrequency = 25
                    Ctrl.Value = 127
                End If
            Next

        Catch ex As Exception
        End Try

    End Sub

    Public Sub LeerPosiciones()
        Try
            Dim cc As Integer
            Dim Valor As Byte
            'Inicializamos la variable de la posicion
            For cc = 0 To 27
                Servo(cc) = 1
                If ModoRS232 = True Then
                    Valor = ModuloRs232.Leer_Posicion(cc)
                    ' Si es un valor valido lo establece a Servo
                    If Valor > 0 Then Servo(cc) = Valor
                Else
                    If MyDeviceDetected Then
                        Servo(cc) = ModuloUSB.Leer_Posicion(cc)
                    End If
                End If
            Next cc


            'Actualizamos los faders
            TrackBar1.Value = Servo(0)
            TrackBar2.Value = Servo(1)
            TrackBar3.Value = Servo(2)
            TrackBar4.Value = Servo(3)
            TrackBar5.Value = Servo(4)
            TrackBar6.Value = Servo(5)
            TrackBar7.Value = Servo(6)
            TrackBar8.Value = Servo(7)
            TrackBar9.Value = Servo(8)
            TrackBar10.Value = Servo(9)
            TrackBar11.Value = Servo(10)
            TrackBar12.Value = Servo(11)
            TrackBar13.Value = Servo(12)
            TrackBar14.Value = Servo(13)
            TrackBar15.Value = Servo(14)
            TrackBar16.Value = Servo(15)
            TrackBar17.Value = Servo(16)
            TrackBar18.Value = Servo(17)
            TrackBar19.Value = Servo(18)
            TrackBar20.Value = Servo(19)
            TrackBar21.Value = Servo(20)
            TrackBar22.Value = Servo(21)
            TrackBar23.Value = Servo(22)
            TrackBar24.Value = Servo(23)
            TrackBar25.Value = Servo(24)
            TrackBar26.Value = Servo(25)
            TrackBar27.Value = Servo(26)
            TrackBar28.Value = Servo(27)

            'Actualizamos los labels de posicion
            Call PresentarLabels()
        Catch ex As Exception
            Call HandleException(Me.Name, ex)
        End Try
    End Sub

    Private Sub TrackBar1_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles _
          TrackBar1.MouseDown, TrackBar2.MouseDown, TrackBar3.MouseDown, TrackBar4.MouseDown _
        , TrackBar5.MouseDown, TrackBar6.MouseDown, TrackBar7.MouseDown, TrackBar8.MouseDown _
        , TrackBar9.MouseDown, TrackBar10.MouseDown, TrackBar11.MouseDown, TrackBar12.MouseDown _
        , TrackBar13.MouseDown, TrackBar14.MouseDown, TrackBar15.MouseDown, TrackBar16.MouseDown _
        , TrackBar17.MouseDown, TrackBar18.MouseDown, TrackBar19.MouseDown, TrackBar20.MouseDown _
        , TrackBar21.MouseDown, TrackBar22.MouseDown, TrackBar23.MouseDown, TrackBar24.MouseDown _
        , TrackBar25.MouseDown, TrackBar26.MouseDown, TrackBar27.MouseDown, TrackBar28.MouseDown _
        , TrackBar29.MouseDown, TrackBar30.MouseDown

        Select Case e.Button
            'Case MouseButtons.Left
            Case Windows.Forms.MouseButtons.Right
                DirectCast(sender, TrackBar).Value = 127

                Dim Index As Integer

                Select Case DirectCast(sender, TrackBar).Name
                    Case TrackBar1.Name : Index = 0
                    Case TrackBar2.Name : Index = 1
                    Case TrackBar3.Name : Index = 2
                    Case TrackBar4.Name : Index = 3
                    Case TrackBar5.Name : Index = 4
                    Case TrackBar6.Name : Index = 5
                    Case TrackBar7.Name : Index = 6
                    Case TrackBar8.Name : Index = 7
                    Case TrackBar9.Name : Index = 8
                    Case TrackBar10.Name : Index = 9
                    Case TrackBar11.Name : Index = 10
                    Case TrackBar12.Name : Index = 11
                    Case TrackBar13.Name : Index = 12
                    Case TrackBar14.Name : Index = 13
                    Case TrackBar15.Name : Index = 14
                    Case TrackBar16.Name : Index = 15
                    Case TrackBar17.Name : Index = 16
                    Case TrackBar18.Name : Index = 17
                    Case TrackBar19.Name : Index = 18
                    Case TrackBar20.Name : Index = 19
                    Case TrackBar21.Name : Index = 20
                    Case TrackBar22.Name : Index = 21
                    Case TrackBar23.Name : Index = 22
                    Case TrackBar24.Name : Index = 23
                    Case TrackBar25.Name : Index = 24
                    Case TrackBar26.Name : Index = 25
                    Case TrackBar27.Name : Index = 26
                    Case TrackBar28.Name : Index = 27
                    Case TrackBar29.Name : Index = 28
                    Case TrackBar30.Name : Index = 29
                End Select

                ' Actualizamos la variable de posicion
                Servo(Index) = CByte(DirectCast(sender, TrackBar).Value)
                ' Si el checkbox esta activado no trabajaran los faders
                If CheckBox1.Checked = False Then
                    If ModoRS232 Then
                        Call ModuloRs232.Posicion_Servo(Index, CByte(DirectCast(sender, TrackBar).Value))
                    Else
                        Call ModuloUSB.Posicion_Servo(Index, CByte(DirectCast(sender, TrackBar).Value))
                    End If
                End If

                'Actualizamos los labels de posicion
                Call PresentarLabels()
        End Select
    End Sub
    Private Sub TrackBar1_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) _
    Handles TrackBar1.Scroll, TrackBar2.Scroll, TrackBar3.Scroll, TrackBar4.Scroll, TrackBar5.Scroll _
    , TrackBar6.Scroll, TrackBar7.Scroll, TrackBar8.Scroll, TrackBar9.Scroll, TrackBar10.Scroll _
    , TrackBar11.Scroll, TrackBar12.Scroll, TrackBar13.Scroll, TrackBar14.Scroll, TrackBar15.Scroll _
    , TrackBar16.Scroll, TrackBar17.Scroll, TrackBar18.Scroll, TrackBar19.Scroll, TrackBar20.Scroll _
    , TrackBar21.Scroll, TrackBar22.Scroll, TrackBar23.Scroll, TrackBar24.Scroll, TrackBar25.Scroll _
    , TrackBar26.Scroll, TrackBar27.Scroll, TrackBar28.Scroll, TrackBar29.Scroll, TrackBar30.Scroll

        Dim Index As Integer
        Select Case DirectCast(sender, TrackBar).Name
            Case TrackBar1.Name : Index = 0
            Case TrackBar2.Name : Index = 1
            Case TrackBar3.Name : Index = 2
            Case TrackBar4.Name : Index = 3
            Case TrackBar5.Name : Index = 4
            Case TrackBar6.Name : Index = 5
            Case TrackBar7.Name : Index = 6
            Case TrackBar8.Name : Index = 7
            Case TrackBar9.Name : Index = 8
            Case TrackBar10.Name : Index = 9
            Case TrackBar11.Name : Index = 10
            Case TrackBar12.Name : Index = 11
            Case TrackBar13.Name : Index = 12
            Case TrackBar14.Name : Index = 13
            Case TrackBar15.Name : Index = 14
            Case TrackBar16.Name : Index = 15
            Case TrackBar17.Name : Index = 16
            Case TrackBar18.Name : Index = 17
            Case TrackBar19.Name : Index = 18
            Case TrackBar20.Name : Index = 19
            Case TrackBar21.Name : Index = 20
            Case TrackBar22.Name : Index = 21
            Case TrackBar23.Name : Index = 22
            Case TrackBar24.Name : Index = 23
            Case TrackBar25.Name : Index = 24
            Case TrackBar26.Name : Index = 25
            Case TrackBar27.Name : Index = 26
            Case TrackBar28.Name : Index = 27
            Case TrackBar29.Name : Index = 28
            Case TrackBar30.Name : Index = 29
        End Select
        ' Actualizamos la variable de posicion
        Servo(Index) = CByte(DirectCast(sender, TrackBar).Value)
        ' Si el checkbox esta activado no trabajaran los faders
        If CheckBox1.Checked = False Then
            If ModoRS232 Then
                Call ModuloRs232.Posicion_Servo(Index, CByte(DirectCast(sender, TrackBar).Value))
            Else
                Call ModuloUSB.Posicion_Servo(Index, CByte(DirectCast(sender, TrackBar).Value))
            End If
        End If

        'Actualizamos los labels de posicion
        Call PresentarLabels()
    End Sub

    Private Sub PresentarLabels()
        ' Presentamos el valor de cada fader
        L1.Text = Trim(Str(Servo(0)))
        L2.Text = Trim(Str(Servo(1)))
        L3.Text = Trim(Str(Servo(2)))
        L4.Text = Trim(Str(Servo(3)))
        L5.Text = Trim(Str(Servo(4)))
        L6.Text = Trim(Str(Servo(5)))
        L7.Text = Trim(Str(Servo(6)))
        L8.Text = Trim(Str(Servo(7)))
        L9.Text = Trim(Str(Servo(8)))
        L10.Text = Trim(Str(Servo(9)))
        L11.Text = Trim(Str(Servo(10)))
        L12.Text = Trim(Str(Servo(11)))
        L13.Text = Trim(Str(Servo(12)))
        L14.Text = Trim(Str(Servo(13)))
        L15.Text = Trim(Str(Servo(14)))
        L16.Text = Trim(Str(Servo(15)))
        L17.Text = Trim(Str(Servo(16)))
        L18.Text = Trim(Str(Servo(17)))
        L19.Text = Trim(Str(Servo(18)))
        L20.Text = Trim(Str(Servo(19)))
        L21.Text = Trim(Str(Servo(20)))
        L22.Text = Trim(Str(Servo(21)))
        L23.Text = Trim(Str(Servo(22)))
        L24.Text = Trim(Str(Servo(23)))
        L25.Text = Trim(Str(Servo(24)))
        L26.Text = Trim(Str(Servo(25)))
        L27.Text = Trim(Str(Servo(26)))
        L28.Text = Trim(Str(Servo(27)))
        L29.Text = Trim(Str(Servo(28)))
        L30.Text = Trim(Str(Servo(29)))
    End Sub


    Private Sub LinkLabel1_LinkClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles LinkLabel1.LinkClicked
        ' Ejecutamos la web
        System.Diagnostics.Process.Start(e.Link.LinkData.ToString())
    End Sub



    Private Sub RadioButton1_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles RadioButton1.Click, RadioButton2.Click
        Try
 
            Select Case DirectCast(sender, RadioButton).Name
                Case RadioButton1.Name : ModoRS232 = False
                Case RadioButton2.Name : ModoRS232 = True
            End Select


            ' Segun el modo de comunicacion abrimos el periferico
            If ModoRS232 Then
                'Grabamos un 1=Modo RS232
                mINI.IniWrite(ArchivoIni, "Settings", "Modo", "1")
                'Abrimos el puerto serie
                Call ModuloRs232.AbrirPuerto()
            Else
                'Grabamos un 0=Modo USB
                mINI.IniWrite(ArchivoIni, "Settings", "Modo", "0")

                'Cerramos los dispositivos USB
                Call ModuloUSB.Close_USB()

                'Abrimos el HID y buscamos la controladora
                Call ModuloUSB.AbrirUSB(True)
            End If
            ' Leemos las posiciones
            Call LeerPosiciones()

        Catch ex As Exception
            Call HandleException(Me.Name, ex)
        End Try
    End Sub


    Private Sub TimerOut_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TimerOut.Tick
        ' Timer de timeOut
        TimerOut.Enabled = False
        TimeOut = True
    End Sub

    Private Sub RadioButton1_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RadioButton1.CheckedChanged

    End Sub

    Private Sub RadioButton2_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RadioButton2.CheckedChanged

    End Sub
End Class