

Module ModuloRs232
    ' Funcin que crea un timer
    Declare Function SetTimer Lib "user32" (ByVal hwnd As Long, ByVal nIDEvent As Long, ByVal uElapse As Long, ByVal lpTimerFunc As Long) As Long
    ' Funcin que detiene el timer iniciado
    Declare Function KillTimer Lib "user32" (ByVal hwnd As Long, ByVal nIDEvent As Long) As Long

    Dim Retraso As Boolean

    Public Sub AbrirPuerto()
        Try
            PuertoAbierto = False

            'Cerramos el puerto si esta abierto
            If frmMain.SerialPort1.IsOpen Then frmMain.SerialPort1.Close()

            'Carganmos la configuracion del puerto
            frmMain.SerialPort1.PortName = mINI.IniGet(ArchivoIni, "Settings", "Puerto", "COM1")
            frmMain.SerialPort1.BaudRate = Val(mINI.IniGet(ArchivoIni, "Settings", "Baudios", "9600"))
            frmMain.SerialPort1.Parity = 0
            frmMain.SerialPort1.DataBits = 8
            frmMain.SerialPort1.StopBits = 1
            frmMain.SerialPort1.Handshake = 0

            ' Esto hace que los caracteres nulos no se tengan en cuenta
            frmMain.SerialPort1.DiscardNull = True

            'Cargamos el metodo por defecto para la conversion ascii
            frmMain.SerialPort1.Encoding = System.Text.Encoding.Default

            frmMain.SerialPort1.RtsEnable = True
            frmMain.SerialPort1.Open()
            PuertoAbierto = True
        Catch ex As Exception
            Call frmMain.HandleException(frmMain.Name, ex)
        End Try

    End Sub
    Public Function Posicion_Servo(ByVal nServo As Integer, ByVal Posicion As Byte) As Byte
        Try
            ' Enviamos el comando "S" al servo con la posicion
            frmMain.SerialPort1.Write("S" + Chr(nServo) + Chr(Posicion), 0, 3)


        Catch ex As Exception
            Call frmMain.HandleException("Posicion_Servo", ex)
        End Try

    End Function

    'Sub UseThread()
    'Dim t As New System.Threading.Thread(AddressOf CountSheep)
    '    t.Start()
    'End Sub

    Public Function Leer_Posicion(ByVal nServo As Integer) As Byte
        'Leemos la posicion de los servos
        Try
            Dim Dato As Byte
            ' indicador de que se lee el primer dato (posicion)
            ' el segundo dato recibido es el 13 (VbCr)
            Dim BitPar As Boolean

            ' Enviamos el comando "S" al servo con la posicion
            frmMain.SerialPort1.Write("P" + Chr(nServo), 0, 2)

            'Creamos un timer de 15 ms para que se envien los datos
            System.Threading.Thread.Sleep(15)

            TimeOut = False
            frmMain.TimerOut.Enabled = True
            ' Esperamos a que salgan los bytes
            While frmMain.SerialPort1.BytesToWrite > 0 And TimeOut = False
                Application.DoEvents()
            End While
            ' Esperamos a que entre la respuesta
            While frmMain.SerialPort1.BytesToRead = 0 And TimeOut = False
                Application.DoEvents()
            End While

            If frmMain.SerialPort1.BytesToRead > 0 Then
                BitPar = False
                Do
                    ' Esto permite inspeccionar otros eventos
                    System.Windows.Forms.Application.DoEvents()

                    Dato = frmMain.SerialPort1.ReadByte

                    If BitPar = False Then
                        BitPar = True
                        If Dato > 0 Then Leer_Posicion = Dato
                    End If
                Loop Until Dato = 13 Or TimeOut = True
                frmMain.TimerOut.Enabled = False
            End If
        Catch ex As Exception
            'Call frmMain.HandleException(frmMain.Name, ex)
        End Try

    End Function


    Public Sub Posicion_All_Servos()
        Try
            ' Enviamos el comando "T" correspondiente a Todos, y seguidamente
            ' las 28 posiciones
            frmMain.SerialPort1.Write("T" + Chr(Servo(0)) + Chr(Servo(1)) _
            + Chr(Servo(2)) + Chr(Servo(3)) + Chr(Servo(4)) + Chr(Servo(5)) _
            + Chr(Servo(6)) + Chr(Servo(7)) + Chr(Servo(8)) + Chr(Servo(9)) _
            + Chr(Servo(10)) + Chr(Servo(11)) + Chr(Servo(12)) + Chr(Servo(13)) _
            + Chr(Servo(14)) + Chr(Servo(15)) + Chr(Servo(16)) + Chr(Servo(17)) _
            + Chr(Servo(18)) + Chr(Servo(19)) + Chr(Servo(20)) + Chr(Servo(21)) _
            + Chr(Servo(22)) + Chr(Servo(23)) + Chr(Servo(24)) + Chr(Servo(25)) _
            + Chr(Servo(26)) + Chr(Servo(27)), 0, 29)

            TimeOut = False
            frmMain.TimerOut.Enabled = True
            ' Esperamos a que salgan los bytes
            While frmMain.SerialPort1.BytesToWrite > 0 And TimeOut = False
                Application.DoEvents()
            End While

        Catch ex As Exception
            Call frmMain.HandleException("Posicion_All_Servos", ex)
        End Try

    End Sub


    Public Sub Comando_Inicio(ByVal nServo As Integer, ByVal Posicion As Byte)
        Try
            ' Enviamos la posicion de inicio al sevo seleccionado
            frmMain.SerialPort1.Write("I" + Chr(nServo) + Chr(Posicion), 0, 3)

            TimeOut = False
            frmMain.TimerOut.Enabled = True
            ' Esperamos a que salgan los bytes
            While frmMain.SerialPort1.BytesToWrite > 0 And TimeOut = False
                Application.DoEvents()
            End While
        Catch ex As Exception
            Call frmMain.HandleException("Comando_Inicio", ex)
        End Try

    End Sub

    Public Function Velocidad_Port(ByVal Velocidad As Byte) As Byte
        'Escribimos a velocidad del puerto serie
        Try
            ' Comprobamos si el puerto esta abierto, si no...salimos
            If frmMain.SerialPort1.IsOpen = False Then Exit Function

            Dim Dato As Byte

            ' Enviamos el comando "A" para asignar la velocidad de comunicacion

            ' 0 - 9.600
            ' 1 - 19.200
            ' 2 - 38.400
            ' 3 - 57.600
            ' 4 - 115.200

            frmMain.SerialPort1.Write("A" + Chr(Velocidad), 0, 2)

            TimeOut = False
            frmMain.TimerOut.Enabled = True
            ' Esperamos a que salgan los bytes
            While frmMain.SerialPort1.BytesToWrite > 0 And TimeOut = False
                Application.DoEvents()
            End While
            ' Esperamos a que entre la respuesta
            While frmMain.SerialPort1.BytesToRead = 0 And TimeOut = False
                Application.DoEvents()
            End While

            If frmMain.SerialPort1.BytesToRead > 0 Then
                Do
                    ' Permite otros eventos
                    System.Windows.Forms.Application.DoEvents()

                    Dato = frmMain.SerialPort1.ReadByte
                Loop Until Dato = 13 Or TimeOut = True
                frmMain.TimerOut.Enabled = False
            End If
        Catch ex As Exception
            Call frmMain.HandleException("Velocidad_Port", ex)
        End Try

    End Function
End Module
